<?php
/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Esther Brunner <wikidesign@gmail.com>
 */

// must be run within Dokuwiki
if(!defined('DOKU_INC')) die();

if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'action.php');

if (!defined('NL')) define('NL',"\n");

class action_plugin_discussion extends DokuWiki_Action_Plugin{

  /**
   * Return some info
   */
  function getInfo(){
    return array(
      'author' => 'Esther Brunner',
      'email'  => 'wikidesign@gmail.com',
      'date'   => '2006-08-28',
      'name'   => 'Discussion Plugin',
      'desc'   => 'Enables discussion features',
      'url'    => 'http://wiki:splitbrain.org/plugin:discussion',
    );
  }

  /**
   * Register the eventhandlers
   */
  function register(&$contr){
    $contr->register_hook('ACTION_ACT_PREPROCESS',
                          'BEFORE',
                          $this,
                          'handle_act_preprocess',
                           array());
                              
    $contr->register_hook('TPL_ACT_RENDER',
                          'AFTER',
                          $this,
                          'comments',
                          array());
  }
  
  /**
   * Main function; dispatches the comment actions
   */
  function comments(&$event, $param){
    if ($event->data != 'show') return; // nothing to do for us
    
    $cid  = $_REQUEST['cid'];
    
    switch ($_REQUEST['comment']){
      
      case 'add':
        $user = $_REQUEST['user'];
        $name = $_REQUEST['name'];
        $mail = $_REQUEST['mail'];
        $raw  = cleanText($_REQUEST['text']);
        $repl = $_REQUEST['reply'];
        $this->_add($user, $name, $mail, $raw, $repl);
        break;
      
      case 'edit':
        $this->_show(NULL, $cid);
        break;
      
      case 'save':
        $raw  = cleanText($_REQUEST['text']);
        $this->_save($cid, $raw);
        break;
        
      case 'toogle':
        $this->_save($cid, '', true);
        break;
            
      default: // 'show' => $this->_show(), 'reply' => $this->_show($cid)
        $this->_show($cid);
    }
  }
  
  /**
   * Shows all comments of the current page
   */
  function _show($reply = NULL, $edit = NULL){
    global $ID;
    
    // get discussion meta file name
    $file = metaFN($ID, '.comments');
    
    if (!file_exists($file)) return true;  // no comments at all
    
    $data = unserialize(io_readFile($file, false));
    
    if ($data['status'] == 0) return true; // comments are off
    
    // section title
    $title = $this->getLang('discussion');
    $secid = cleanID($title);
    echo '<h2><a name="'.$secid.'" id="'.$secid.'">'.$title.'</a></h2>';
    echo '<div class="level2">';
        
    // now display the comments
    if (isset($data['comments'])){
      foreach ($data['comments'] as $key => $value){
        if ($key == $edit) $this->_form($value['raw'], 'save', $edit); // edit form
        else $this->_print($key, $data, '', $reply);
      }
    }
    
    // comment form
    if (($data['status'] == 1) && !$reply && !$edit) $this->_form('');
    
    echo '</div>';
    
    return true;
  }
  
  /**
   * Adds a new comment and then displays all comments
   */
  function _add($user, $name, $mail, $raw, $parent){
    global $ID;
    global $TEXT;
    
    $TEXT = $raw;
    
    // spamcheck against the DokuWiki blacklist
    if (checkwordblock()){
      msg($this->getLang('wordblock'), -1);
      $this->_show();
      return false;
    }
    
    // get discussion meta file name
    $file = metaFN($ID, '.comments');
    
    $data = array();
    $data = unserialize(io_readFile($file, false));
    
    if ($data['status'] != 1) return false;                // comments off or closed
    if ((!$this->getConf('allowguests'))
      && ($user != $_SERVER['REMOTE_USER'])) return false; // guest comments not allowed
    
    $date = time();
    $cid  = md5($user.$date);                              // create a unique id
    
    if (!is_array($data['comments'][$parent])) $parent = NULL; // invalid parent comment
    
    // render the comment
    require_once(DOKU_INC.'inc/parserutils.php');
    $xhtml = p_render('xhtml', p_get_instructions($raw), $info);
    
    // fill in the new comment
    $data['comments'][$cid] = array(
      'user'    => htmlspecialchars($user),
      'name'    => htmlspecialchars($name),
      'mail'    => htmlspecialchars($mail),
      'date'    => $date,
      'show'    => true,
      'raw'     => trim($raw),
      'xhtml'   => $xhtml,
      'parent'  => $parent,
      'replies' => array()
    );
    
    // update parent comment
    if ($parent) $data['comments'][$parent]['replies'][] = $cid;
    
    // update the number of comments
    $data['number']++;
        
    // save the comment metadata file
    io_saveFile($file, serialize($data));
    
    // notify subscribers of the page
    $this->_notify($data['comments'][$cid]);
    
    $this->_show();
    return true;
  }
    
  /**
   * Saves the comment with the given ID and then displays all comments
   */
  function _save($cid, $raw, $toogle = false){
    global $ID;
    global $TEXT;
    global $INFO;
    
    $TEXT = $raw;
    
    // spamcheck against the DokuWiki blacklist
    if (checkwordblock()){
      msg($this->getLang('wordblock'), -1);
      $this->_show();
      return false;
    }
    
    // get discussion meta file name
    $file = metaFN($ID, '.comments');
    
    $data = array();
    $data = unserialize(io_readFile($file, false));
        
    // someone else was trying to edit our comment -> abort
    if (($data['comments'][$cid]['user'] != $_SERVER['REMOTE_USER'])
      && ($INFO['perm'] != AUTH_ADMIN)) return false;
    
    if ($toogle){     // toogle visibility
      $now = $data['comments'][$cid]['show'];
      $data['comments'][$cid]['show'] = !$now;
      $data['number'] = $this->_count($data);
      
    } elseif (!$raw){ // remove the comment
      unset($data['comments'][$cid]);
      $data['number'] = $this->_count($data);
            
    } else {          // save changed comment
      $xhtml = $this->render($raw);
      
      // now change the comment's content
      $data['comments'][$cid]['edited'] = time();
      $data['comments'][$cid]['raw']    = trim($raw);
      $data['comments'][$cid]['xhtml']  = $xhtml;
    }
    
    // save the comment metadata file
    io_saveFile($file, serialize($data));
    
    $this->_show();
    return true;
  }
    
  /**
   * Prints an individual comment
   */
  function _print($cid, &$data, $parent = '', $reply = '', $visible = true){
    global $conf;
    global $lang;
    global $ID;
    global $INFO;
    
    $comment = $data['comments'][$cid];
    
    if (!is_array($comment)) return false;          // corrupt datatype
    
    if ($comment['parent'] != $parent) return true; // reply to an other comment
    
    if (!$comment['show']){                         // comment hidden
      if ($INFO['perm'] == AUTH_ADMIN) echo '<div class="comment_hidden">'.NL;
      else return true;
    }
        
    // comment head with date and user data
    echo '<div class="comment_head">'.NL;
    echo '<a name="comment__'.$cid.'" id="comment__'.$cid.'">'.NL;
    
    // show gravatar image
    if ($this->getConf('usegravatar')){
      $default = DOKU_URL.'lib/plugins/discussion/images/default.gif';
      $size    = $this->getConf('gravatar_size');
      if ($comment['mail']) $src = ml('http://www.gravatar.com/avatar.php?'.
        'gravatar_id='.md5($comment['mail']).
        '&default='.urlencode($default).
        '&size='.$size.
        '&rating='.$this->getConf('gravatar_rating'));
      else $src = $default;
      $title = ($comment['name'] ? $comment['name'] : obfuscate($comment['mail']));
      echo '<img src="'.$src.'" class="medialeft" title="'.$title.'"'.
        ' alt="'.$title.'" width="'.$size.'" height="'.$size.'" />'.NL;
    }
    
    echo '</a>'.NL;
    if ($this->getConf('linkemail') && $comment['mail']){
      echo '<a href="mailto:'.obfuscate($comment['mail']).'">'.$comment['name'].'</a>';
    } else {
      echo $comment['name'];
    }
    echo ', '.date($conf['dformat'], $comment['date']);
    if ($comment['edited']) echo ' ('.date($conf['dformat'], $comment['edited']).')';
    echo ':'.NL;
    echo '</div>'.NL;
    
    // main comment content
    echo '<div class="comment_body">'.NL;
    echo $comment['xhtml'].NL;
    echo '</div>'.NL;
    
    
    if ($visible){
      // show hide/show toogle button?
      echo '<div class="comment_buttons">'.NL;
      if ($INFO['perm'] == AUTH_ADMIN){
        if (!$comment['show']) $label = $this->getLang('btn_show');
        else $label = $this->getLang('btn_hide');
        
        $this->_button($cid, $label, 'toogle');
      }
          
      // show reply button?
      if (($data['status'] == 1) && !$reply && $comment['show'])
        $this->_button($cid, $this->getLang('btn_reply'), 'reply');
      
      // show edit button?
      if (($comment['user'] == $_SERVER['REMOTE_USER']) || ($INFO['perm'] == AUTH_ADMIN))
        $this->_button($cid, $lang['btn_secedit'], 'edit');
      echo '</div>'.NL;
    }

    // replies to this comment entry?
    if (count($comment['replies'])){
      echo '<div class="comment_replies">'.NL;
      $visible = ($comment['show'] && $visible);
      foreach ($comment['replies'] as $rid){
        $this->_print($rid, $data, $cid, $reply, $visible);
      }
      echo '</div>'.NL;
    }
    
    if (!$comment['show']) echo '</div>'.NL; // class="comment_hidden"
    
    // reply form
    if ($reply == $cid){
      echo '<div class="comment_replies">'.NL;
      $this->_form('', 'add', $cid);
      echo '</div>'.NL;
    }
  }

  /**
   * Outputs the comment form
   */
  function _form($raw = '', $act = 'add', $cid = NULL){
    global $lang;
    global $conf;
    global $ID;
    global $INFO;
  
    // not for unregistered users when guest comments aren't allowed
    if (!$_SERVER['REMOTE_USER'] && !$this->getConf('allowguests')) return false;
    
    ?>
    <div class="comment_form">
      <form id="discussion__comment_form" method="post" action="<?php echo script() ?>" accept-charset="<?php echo $lang['encoding'] ?>" onsubmit="return validate(this);">
        <div class="no">
          <input type="hidden" name="id" value="<?php echo $ID ?>" />
          <input type="hidden" name="do" value="show" />
          <input type="hidden" name="comment" value="<?php echo $act ?>" />
    <?php
    
    // for adding a comment
    if ($act == 'add'){
      ?>
          <input type="hidden" name="reply" value="<?php echo $cid ?>" />
      <?php
      // for registered user
      if ($conf['useacl'] && $_SERVER['REMOTE_USER']){
      ?>
          <input type="hidden" name="user" value="<?php echo $_SERVER['REMOTE_USER'] ?>" />
          <input type="hidden" name="name" value="<?php echo $INFO['userinfo']['name'] ?>" />
          <input type="hidden" name="mail" value="<?php echo $INFO['userinfo']['mail'] ?>" />
      <?php
      // for guest: show name and e-mail entry fields
      } else {
      ?>
          <input type="hidden" name="user" value="<?php echo clientIP() ?>" />
          <div class="comment_name">
            <label class="block" for="discussion__comment_name">
              <span><?php echo $lang['fullname'] ?>:</span>
              <input type="text" class="edit" name="name"' id="discussion__comment_name" size="50" tabindex="1" />
            </label>
          </div>
          <div class="comment_mail">
            <label class="block" for="discussion__comment_mail">
              <span><?php echo $lang['email'] ?>:</span>
              <input type="text" class="edit" name="mail" id="discussion__comment_mail" size="50" tabindex="2" />
            </label>
          </div>
      <?php
      }
      
    // for saving a comment
    } else {
    ?>
          <input type="hidden" name="cid" value="<?php echo $cid ?>" />
    <?php
    }
    ?>
          <div class="comment_text">
            <textarea class="edit" name="text" cols="80" rows="10" id="discussion__comment_text" tabindex="3"><?php echo $raw ?></textarea>
          </div>
          <input class="button" type="submit" name="submit" value="<?php echo $lang['btn_save'] ?>" tabindex="4" />
        </div>
      </form>
    </div>
    <?php
    echo $this->_coComment();
  }
  
  /**
   * Adds a javascript to interact with coComments
   */
  function _coComment(){
    global $ID;
    global $conf;
    global $INFO;
    
    $user = $_SERVER['REMOTE_USER'];
    
    ?>
    <script type="text/javascript"><!--//--><![CDATA[//><!--
      var blogTool  = "DokuWiki";
      var blogURL   = "<?php echo DOKU_URL ?>";
      var blogTitle = "<?php echo $conf['title'] ?>";
      var postURL   = "<?php echo wl($ID, '', true) ?>";
      var postTitle = "<?php echo tpl_pagetitle($ID, true) ?>";
    <?php
    if ($user){
    ?>
      var commentAuthor = "<?php echo $INFO['userinfo']['name'] ?>";
    <?php
    } else {
    ?>
      var commentAuthorFieldName = "name";
    <?php
    }
    ?>
      var commentAuthorLoggedIn = <?php echo ($user ? 'true' : 'false') ?>;
      var commentFormID         = "discussion__comment_form";
      var commentTextFieldName  = "text";
      var commentButtonName     = "submit";
      var cocomment_force       = false;
    //--><!]]></script>
    <script type="text/javascript" scr="http://www.cocomment.com/js/cocomment.js">
    </script>
    <?php
  }
    
  /**
   * General button function
   */
  function _button($cid, $label, $act){
    global $ID;
        
    ?>
    <form class="button" method="post" action="<?php echo script() ?>">
      <div class="no">
        <input type="hidden" name="id" value="<?php echo $ID ?>" />
        <input type="hidden" name="do" value="show" />
        <input type="hidden" name="comment" value="<?php echo $act ?>" />
        <input type="hidden" name="cid" value="<?php echo $cid ?>" />
        <input type="submit" value="<?php echo $label ?>" class="button" title="<?php echo $label ?>" />
      </div>
    </form>
    <?php
    return true;
  }
  
  /**
   * Sends a notify mail on new comment
   *
   * @param  array  $comment  data array of the new comment
   *
   * @author Andreas Gohr <andi@splitbrain.org>
   * @author Esther Brunner <wikidesign@gmail.com>
   */
  function _notify($comment){
    global $conf;
    global $ID;
  
    if (!$conf['subscribers']) return; //subscribers enabled?
    $bcc  = subscriber_addresslist($ID);
    if (empty($bcc)) return;
    $to   = '';
    $text = io_readFile($this->localFN('subscribermail'));
  
    $text = str_replace('@PAGE@', $ID, $text);
    $text = str_replace('@TITLE@', $conf['title'], $text);
    $text = str_replace('@DATE@', date($conf['dformat'], $comment['date']), $text);
    $text = str_replace('@NAME@', $comment['name'], $text);
    $text = str_replace('@TEXT@', $comment['raw'], $text);
    $text = str_replace('@UNSUBSCRIBE@', wl($ID, 'do=unsubscribe', true, '&'), $text);
    $text = str_replace('@DOKUWIKIURL@', DOKU_URL, $text);
  
    $subject = '['.$conf['title'].'] '.$this->getLang('mail_newcomment');
  
    mail_send($to, $subject, $text, $conf['mailfrom'], '', $bcc);
  }
  
  /**
   * Counts the number of visible comments
   */
  function _count($data){
    $number = 0;
    foreach ($data['comments'] as $cid => $comment){
      if ($comment['parent']) continue;
      if (!$comment['show']) continue;
      $number++;
      $rids = $comment['replies'];
      if (count($rids)) $number = $number + $this->_countReplies($data, $rids);
    }
    return $number;
  }
  
  function _countReplies(&$data, $rids){
    $number = 0;
    foreach ($rids as $rid){
      if (!$data['comments'][$rid]['show']) continue;
      $number++;
      $rids = $data['comments'][$rid]['replies'];
      if (count($rids)) $number = $number + $this->_countReplies($data, $rids);
    }
    return $number;
  }
    
  /**
   * Checks if 'newthread' was given as action, if so we
   * do handle the event our self and no further checking takes place
   */
  function handle_act_preprocess(&$event, $param){
    if ($event->data != 'newthread') return; // nothing to do for us
    
    global $ACT;
    global $ID;

    // we can handle it -> prevent others
    $event->stopPropagation();
    $event->preventDefault();
    
    $ns    = $_REQUEST['ns'];
    $title = str_replace(':', '', $_REQUEST['title']);
    $id    = ($ns ? $ns.':' : '').cleanID($title);
    
    // check if we are allowed to create this file
    if (auth_quickaclcheck($id) >= AUTH_CREATE){
      $back = $ID;
      $ID   = $id;
      $file = wikiFN($ID);
      
      //check if locked by anyone - if not lock for my self      
      if (checklock($ID)){
        $ACT = 'locked';
      } else {
        lock($ID);
      }

      // prepare the new thread file with default stuff
      if (!@file_exists($file)){
        global $TEXT;
        global $INFO;
        global $conf;
        
        $TEXT = pageTemplate($ns.':'.$title);
        if (!$TEXT) $TEXT = "<- [[:$back]]\n\n====== $title ======\n\n".
                            "{{gravatar>".$INFO['userinfo']['mail']." }} ".
                            "//".$INFO['userinfo']['name'].", ".
                            date($conf['dformat']).": //\n\n\n\n".
                            "~~DISCUSSION~~\n";
        $ACT = 'preview';
      } else {
        $ACT = 'edit';
      }
    } else {
      $ACT = 'show';
    }
  }

}

//Setup VIM: ex: et ts=4 enc=utf-8 :
